; ----------------------------------------------------------------------------
#include "defs.h"

; ----------------------------------------------------------------------------
.text
; ----------------------------------------------------------------------------
*   = FINAL_ADDRESS_P8D_LOADER

    sei
    cld

    ; Switch to HIRES
    lda   #$40
    ldy   #>($c000-$9800)
    ldx   #0
loop1
__auto_hires
    sta   $9800,x
    inx   
    bne   loop1
    inc   __auto_hires+2
    dey   
    bne   loop1

    lda   #26
    sta   $bf40
    lda   #30
    sta   $bfdf
xxx       
    jsr   PreNibblize
yyy       
    jsr   load_loader
zzz       
    jmp   zzz

dest      =     $76
desthi    =     $77
sect      =     $78
track     =     $79
tracklast =     $7a

wrk0      =     $7b
wrk1      =     $7c
wrk2      =     $7d


#define FLOPPY_SECTOR_PER_TRACK 16

#define FB_ENTRY_1_TRACK        30
#define FB_ENTRY_1_SECTOR       15
#define FB_ENTRY_1_LENGTH       32

#define hires_screen            $a000
#define bffr                    $6000

loader_8d_track  .byt FB_ENTRY_1_TRACK
loader_8d_sector .byt FB_ENTRY_1_SECTOR
loader_8d_length .byt FB_ENTRY_1_LENGTH

pic_counter      .byt 14

; ----------------------------------------------------------------------------
load_loader
          lda   #$00
          sta   track
          sta   tracklast
          
loader_loop
          lda   #<hires_screen
          sta   dest
          lda   #>hires_screen
          sta   desthi

read_sector_seek
          lda   loader_8d_track
          sta   track
          jsr   Seek

read_sector_loop
          ldx   loader_8d_sector
          lda   intrlv,x
          sta   sect
          jsr   Read_Sector
          
          dec   loader_8d_length
          bne   next_sector

          lda   #FB_ENTRY_1_SECTOR
          sta   loader_8d_sector
          
          lda   #FB_ENTRY_1_LENGTH
          sta   loader_8d_length
          
          dec   loader_8d_track
          
          dec   pic_counter
          bne   loader_loop
          
          lda   #14
          sta   pic_counter
          
          lda   #FB_ENTRY_1_TRACK
          sta   loader_8d_track

          jmp   loader_loop
          
          bit   drvoff
          rts

next_sector
          inc   desthi
          dec   loader_8d_sector
          bpl   read_sector_loop
          lda   #(FLOPPY_SECTOR_PER_TRACK-1)
          sta   loader_8d_sector
          dec   loader_8d_track
          jmp   read_sector_seek

          
; ----------------------------------------------------------------------------
; Create the nibble table
PreNibblize
          ldy   #$00
          ldx   #$03
nib_loop
          stx   wrk0
          txa
          asl
          bit   wrk0
          beq   bge
          ora   wrk0
          eor   #$ff
          and   #$7e
ne
          bcs   bge
          lsr
          bne   ne
          tya
          sta   nibble+$56,x
          iny
bge
          inx
          bpl   nib_loop
          rts

; ----------------------------------------------------------------------------
; Seek routine
; A = track
Seek
;
          asl
          sta   wrk0
          cmp   tracklast
          beq   seekdone
          lda   #0
          sta   wrk1
seekloop
          lda   tracklast
          sta   wrk2
          sec
          sbc   wrk0
          beq   seektog2
          bcs   skip0
          eor   #$ff
          inc   tracklast
          bcc   skip1
skip0
          adc   #$fe
          dec   tracklast
skip1
          cmp   wrk1
          bcc   skip2
          lda   wrk1
skip2
          cmp   #$0c
          bcs   skip3
          tay
skip3
          sec
          jsr   seektog1
          lda   tbl_on,y
          jsr   seekdely
          lda   wrk2
          clc
          jsr   seek_tgl
          lda   tbl_off,y
          jsr   seekdely
          inc   wrk1
          bne   seekloop
seektog2
          jsr   seekdely
          clc
seektog1
          lda   tracklast
seek_tgl
          and   #$03
          rol
          tax
          lda   phase,x
seekdone
          rts

seekdely
          ldx   #$13
delyloop  
          dex
          bne   delyloop
          sec
          sbc   #1
          bne   seekdely
          rts

; sector interleave table
intrlv   .byt   $00,$0d,$0b,$09,$07,$05,$03,$01
         .byt   $0e,$0c,$0a,$08,$06,$04,$02,$0f

; acceleration/deceleration tables
tbl_on   .byt   $01,$30,$28,$24,$20,$1e,$1d,$1c,$1c,$1c,$1c,$1c
tbl_off  .byt   $70,$2c,$26,$22,$1f,$1e,$1d,$1c,$1c,$1c,$1c,$1c

gap0 = *
; ============================================================================
                         page
; ----------------------------------------------------------------------------
#echo gap0 free bytes:
#print(* - gap0)
; ----------------------------------------------------------------------------
; Read sector routine
Read_Sector
;
rsect
          clc
rsct1
          php
pl1
          lda   rddata
          bpl   pl1
ne2
          eor   #$d5
          bne   pl1
ne1
          lda   rddata
          bpl   ne1
          cmp   #$aa
          bne   ne2
          nop
pl2
          lda   rddata
          bpl   pl2
          cmp   #$96
          beq   et1
          plp
          bcc   rsect
          eor   #$ad
          beq   lb1
          bne   rsect
et1
          ldy   #3
et2
          sta   wrk1
pl3
          lda   rddata
          bpl   pl3
          rol
          sta   wrk0
pl4
          lda   rddata
          bpl   pl4
          and   wrk0
          dey
          bne   et2
          plp
          cmp   sect
          bne   rsect
          lda   wrk1
          cmp   track
          bne   rsect
          bcs   rsct1
lb1
          ldy   #$56
lb2
          sty   wrk0
pl5
          ldy   rddata
          bpl   pl5
          eor   nibble-$2a,y
          ldy   wrk0
          dey
          sta   nibble,y
          bne   lb2
lb3
          sty   wrk0
pl6
          ldy   rddata
          bpl   pl6
          eor   nibble-$2a,y
          ldy   wrk0

;           sta   (dest),y
          sta   bffr,y
          
          iny
          bne   lb3
pl7
          ldy   rddata
          bpl   pl7
          eor   nibble-$2a,y
          bne   rsect

          ldy   #0
ni
          ldx   #$56
ne3
          dex
          bmi   ni
;           lda   (dest),y
          lda   bffr,y
          lsr   nibble,x
          rol
          lsr   nibble,x
          rol
          sta   (dest),y
          iny
          bne   ne3
          rts
